/*
 * Goldwater Distributed Web Architecture.
 *
 * User Command API
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 *
 * Goldwater is written and maintained by netFluid Technology Ltd.
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.2 $  $Date: 2003/03/29 12:21:11 $
 * 
 * --------------------------------------------------------------------------
 * $Log: gwapi.h,v $
 * Revision 1.2  2003/03/29 12:21:11  csmith
 * cvs merge to 0.3.4
 *
 * Revision 1.11  2002/11/11 20:26:34  chris
 * implemented gw_svc_forward.
 *
 * Revision 1.10  2002/08/24 13:23:16  chris
 * Broke __gw_complete_request into a sub call to __gw_complete_request_explicit
 * which allows the caller to explicitly define which service in which domain
 * the service counters are to be incremented for.
 *
 * Revision 1.9  2002/08/05 00:42:17  chris
 * Changed __gw_set_requestCtrl to take new and pass back copy of old - this
 * allows buffers to be swapped around the internal req_ctrl.
 * ie given parameters &A and &B, Perform: B <- req_ctrl <- A
 *
 * Revision 1.8  2002/08/04 19:36:03  chris
 * query_bb now allows scanning of a single domain's services.
 * API changes to allow replies to be sent back from dombridge.
 *
 * Revision 1.7  2002/06/10 17:34:47  chris
 * All phlib includes have phlib/ prefix and gwlib includes have nothing.
 *
 * Revision 1.6  2002/05/13 19:58:49  chris
 * Additional functionality to support the Domain Bridge.
 * Now allows __gw_get_request to 'not block' on request queue, and to
 * indicate that it returned because no messages were available.
 * More flags to control this behaviour (both control and private_user).
 *
 * Revision 1.5  2002/05/12 14:03:13  chris
 * GPL licence headers etc
 *
 * Revision 1.4  2001/12/26 22:57:43  chris
 * Changed all references to Imagepage Computing to netFluid Technology.
 *
 * Revision 1.3  2001/03/22 17:04:35  chris
 * Added arguments to __gw_get_request prototype.
 *
 * Revision 1.2  2001/03/22 01:16:34  chris
 * Split get_request into three functions.
 *
 * Revision 1.1  2001/03/19 16:03:25  chris
 * Initial revision
 *
 * --------------------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <errno.h>

#include <phlib/xl_types.h>
#include <phlib/xl_string.h>

#include <gwreq.h>
#include <gwportal.h>
#include <gwclient.h>

/* Internal functions */
void __gw_server_init     ( int , char ** );
long __gw_get_request     ( char **f, char **s, char **r, long *fl );
int  __gw_invoke_C_service( );
int  __gw_complete_request( int done );
int  __gw_complete_request_explicit( GWMsgCtrl *, int, const char *, int );

GWMsgCtrl *__gw_get_requestCtrl();
void       __gw_set_requestCtrl( GWMsgCtrl *new, GWMsgCtrl *old );

/* Public functions */
int  gw_send_reply ( GWStatus status, const char *data, long len );
int  gw_svc_forward( const char *service, const char *reqdata, long reqlen, 
		     long flags );

/* ------------------------------------------------------------------------- */
/* end */
