/*
 * Goldwater Distributed Web Architecture.
 *
 * User Client Command API
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 *
 * Goldwater is written and maintained by netFluid Technology Ltd.
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Revision: 1.2 $  $Date: 2003/03/29 12:21:11 $
 * 
 * --------------------------------------------------------------------------
 * $Log: gwclient.h,v $
 * Revision 1.2  2003/03/29 12:21:11  csmith
 * cvs merge to 0.3.4
 *
 * Revision 1.10  2003/03/24 22:12:32  chris
 * Allow forcing of disconnect (ignore reference count).
 *
 * Revision 1.9  2003/01/03 00:09:11  chris
 * Use internal umask when creating files, shm and queues
 *
 * Revision 1.8  2002/11/30 19:17:08  chris
 * Message Buffer handling
 *
 * Revision 1.7  2002/11/10 12:23:41  chris
 * Cleanup and low level API changes for bindings such as C#
 *
 * Revision 1.6  2002/06/02 19:26:42  chris
 * Maintain count of pending requests (requests in queue).
 *
 * Revision 1.5  2002/05/12 14:03:13  chris
 * GPL licence headers etc
 *
 * Revision 1.4  2001/12/26 22:56:51  chris
 * New style of goldwter client config file (uses [SECTION] sections)
 *
 * Revision 1.3  2001/03/22 16:55:28  chris
 * Moved GWStatus to gwtypes.h.  Added gw_release() prototype.
 *
 * Revision 1.2  2001/03/19 17:42:17  chris
 * CFG_* parameters (static const chars) were being included in multiple
 * files.
 *
 * Revision 1.1  2001/03/19 16:04:16  chris
 * Initial revision
 *
 * --------------------------------------------------------------------------
 */
#ifndef _GW_CLIENT_H
#define _GW_CLIENT_H

#include <gwtypes.h>
#include <gwapi.h>

#include <phlib/xl_types.h>
#include <phlib/xl_string.h>

#ifdef GW_PRV_CLIENT

#define RESMAX   8192

static const char GWCONFIG    [] = "GWCONFIG";
static const char CFG_MODE    [] = "mode"   ;
static const char CFG_IPCKEY  [] = "ipckey" ;
static const char CFG_ADDR    [] = "addr" ;
static const char CFG_PORT    [] = "port" ;
static const char CFG_REGISTRY[] = "regaddr";
static const char CFG_LOGNAME [] = "logfile";
static const char CFG_LOGLEVEL[] = "loglevel";
static const char CFG_LOCAL   [] = "local";
static const char CFG_REMOTE  [] = "remote";

static const char CFG_SEC_LOCDOM  [] = "LOCAL_DOMAIN";
static const char CFG_SEC_REGISTRY[] = "REGISTRY";

#endif

static const char MODE_NET    [] = "net";
static const char MODE_SHM    [] = "shm";

/*
 * Client message wrapper.....
 */
typedef struct {
  char *data;     /* Pointer to data */
  long  len;      /* Length of data  */

  /* -- Private Internal Status Caching -- */
  CString _bcx;      /* Buffer info     */
  char    _ctx[128]; /* Temp resources  */ 
  long    _pflg;     /* Private flags   */
} GWMessage;


int  gw_init();
void gw_release( int force );
int  gw_svc_call( const char *, const char *, long, GWMessage *, long );
void gw_free_message( GWMessage * );

__inline int __gw_init();
int  __gw_decrement_queue_count( long idx );
int  __gw_increment_queue_count( long idx );
int  __gw_cs_svc_call( const char *service, const char *reqdata, long reqlen,
                  char **replydata, long *replylen, long flags );
void __gw_cs_svc_call_cleanup();

RetValue   __gw_umask_from_config( int domlev );


#endif
