/*
 * Goldwater Distributed Web Architecture.
 *
 * Log Handling
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 *
 * Goldwater is written and maintained by netFluid Technology Ltd.
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 * $Revision: 1.3 $  $Date: 2003/06/12 11:56:09 $
 * 
 * --------------------------------------------------------------------------
 * $Log: gwlog.h,v $
 * Revision 1.3  2003/06/12 11:56:09  csmith
 * C# wrappers
 *
 * Revision 1.2  2003/03/29 12:21:11  csmith
 * cvs merge to 0.3.4
 *
 * Revision 1.19  2003/01/07 00:29:38  chris
 * Changed phlib linking as we've dropped /lib/phlib ext
 *
 * Revision 1.18  2003/01/05 22:26:34  chris
 * Lots of trickery to do superuser related tasks as quickly and safely as
 * possible before being permanently demoted to 'user'.
 *
 * Revision 1.17  2003/01/02 18:49:56  chris
 * Added User and Group
 *
 * Revision 1.16  2002/11/10 12:23:41  chris
 * Cleanup and low level API changes for bindings such as C#
 *
 * Revision 1.15  2002/10/20 13:38:34  chris
 * Enabled setting of loglevel from administrator (removed _LogLevel and
 * replaced it with *_LogLevelPtr through out code).
 *
 * Revision 1.14  2002/09/01 10:59:31  chris
 * Addition of namespaceing to service name.
 * Service may now be of the form this.is.a.namespace/and/this/is/the/service
 * Wildcards are NOT allowed in RPC type message destinations.
 *
 * Revision 1.13  2002/06/16 11:40:35  chris
 * BB Client is now the Domain Bridge message tracking list.
 * New error codes and messages.
 *
 * Revision 1.12  2002/06/15 18:54:39  chris
 * Domain Bridge additions
 *
 * Revision 1.11  2002/05/30 19:37:34  chris
 * Create entries in BBClient table for each new client attaching to
 * application.
 *
 * Revision 1.10  2002/05/12 22:59:02  chris
 * Added new error GWE_UNSTABLE - returned when GW detected to be in an
 * unstable state.
 *
 * Revision 1.9  2002/05/12 14:03:13  chris
 * GPL licence headers etc
 *
 * Revision 1.8  2002/02/28 00:55:16  chris
 * Various bug fixes.
 *
 * Revision 1.7  2002/02/24 18:05:38  chris
 * Moved APPDIR to BBControl block instead of duplicating it for each and
 * every server.
 * Return GWE_APPERR and set gwusrerr when Services return failure.
 * Use new regstrings that describe the new XML based configuration.
 * Added new functions to support individual addition of service entries.
 * New Version 1.14
 *
 * Revision 1.6  2002/01/03 20:27:31  chris
 * Introduction of preliminary Domain Control handling.
 *
 * Revision 1.5  2001/12/26 22:57:43  chris
 * Changed all references to Imagepage Computing to netFluid Technology.
 *
 * Revision 1.4  2001/03/30 22:16:01  chris
 * Added GWE_BADHDR error and desc string.
 *
 * Revision 1.3  2001/03/22 14:49:00  chris
 * Moved Error level flags to gwtypes.h
 *
 * Revision 1.2  2001/03/22 01:14:55  chris
 * New func gw_log.
 *
 * Revision 1.1  2001/03/19 16:05:36  chris
 * Initial revision
 *
 * --------------------------------------------------------------------------
 */
#ifndef GW_LOGGING_H
#define GW_LOGGING_H

#include <gwtypes.h>

#include <errno.h>

/* Give error value translations....
 */
#ifdef NEED_ERROR_STRINGS
static const char ES_NOFUNC [] = "No function mapped to service";
static const char ES_TIMEOUT[] = "Service call timed out";
static const char ES_GENFAIL[] = "General Application Failure";
static const char ES_NOTCNCT[] = "Client not connected to Goldwater";
static const char ES_READERR[] = "Read error";
static const char ES_GWUAVAL[] = "Goldwater Server Unavailable";
static const char ES_DISCNCT[] = "Unexpected disconnection";
static const char ES_BADDATA[] = "Bad data";
static const char ES_BADCNFG[] = "Bad config entry";
static const char ES_CFGNFND[] = "Config not found";
static const char ES_SYSFAIL[] = "Internal System Failure";
static const char ES_NOSVC  [] = "Service does not exist";
static const char ES_SBLOCKD[] = "Service is blocked from access";
static const char ES_REPSENT[] = "Reply has already been sent";
static const char ES_REPNSNT[] = "Reply was not expected, dispatch ignored";
static const char ES_BADHDR [] = "Bad Header";
static const char ES_SPLDIR [] = "Unable to create/write to spool directory";
static const char ES_APPERR [] = "Application Failure";
static const char ES_CFGRERR[] = "Cannot open configuration";
static const char ES_BCFGARC[] = "Configuration not for this architecture";
static const char ES_BADCFG [] = "Goldwater configuration is invalid";
static const char ES_NOTUP  [] = "Goldwater is not running";
static const char ES_STILLUP[] = "Goldwater is already running";
static const char ES_CLTFAIL[] = "Client could not attach to Goldwater";
static const char ES_UNSTABLE[]= "Goldwater is unstable";
static const char ES_BADDMVER[]= "Invalid Domain Bridge version number";
static const char ES_BADIPCKY[]= "Invalid IPCKEY resource in config";
static const char ES_BADLOGF []= "Invalid Logfile resource in config";
static const char ES_BADDADDR[]= "Invalid/Missing Domain address in config";
static const char ES_BADNSPC []= "Invalid/Unknown Namespace supplied";
static const char ES_BADUSER []= "Invalid user name";
static const char ES_BADGROUP[]= "Invalid user group";
static const char ES_SETUSER []= "Unable to become user";
static const char ES_SETGROUP[]= "Unable to become group";

#endif

#  define gw_log_at_level(x) \
( (*_LogLevelPtr | LOG_ALLSYS) & (long)(x) )

/* prototypes 
 */
void gw_log      ( long level, char *st, int len );
void gw_logf     ( long level, char *fmt, ... );
void gw_closelog ();
void gw_log_reset();
void gw_log_init ( long, const char * );
const char *gw_errorstr( long error );
const char *gw_last_errorstr();
int __gw_cs_get_gwusrerr();
int __gw_cs_get_gwerror();

#endif
