/*
 * Goldwater Distributed Web Architecture.
 *
 * Carrier Message
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 *
 * Goldwater is written and maintained by netFluid Technology Ltd.
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *
 * $Revision: 1.4 $  $Date: 2003/07/20 17:49:08 $
 * --------------------------------------------------------------------------
 */
#ifndef _MSG_CHAIN_H
#define _MSG_CHAIN_H

#include <sys/types.h>
#include <phlib/xl_string.h>

#include <gwclient.h>


#define GWMC_VERSION_V1_0        0x0100
#define GWMC_MAGIC_NUMBER        0xAce1

#define GWMC_MIN_MSG_SIZE 4096

/* Message Header Offsets */
#define GWMC_HDR_MAGIC_NUM_IX    0
#define GWMC_HDR_VERSION_IX      4
#define GWMC_HDR_BODY_OFFSET_IX  8
#define GWMC_HDR_NUM_ITEMS_IX    12
#define GWMC_HDR_TOTAL_LEN_IX    16

#define GWMC_HEADER_LENGTH       20


/* Message block offsets */
#define GWMC_BLK_BLOCKLEN_IX     0
#define GWMC_BLK_MSGID_IX        4
#define GWMC_BLK_OCC_IX          8
#define GWMC_BLK_NEXT_OCC_IX     12
#define GWMC_BLK_CONT_LEN_IX     16
#define GWMC_BLK_DATA_OFFSET_IX  20
#define GWMC_BLK_MIMETYPE_IX     24

#define GWMC_BLK_SIZE            28


typedef struct {
  CString cstr;
  int     items;
  int     error;
  long    last_block_offset;
  GWMessage gwmsg;    /* GWMessage aware to help with garbage collection */
 } MsgChain;

typedef enum {
  GWMCE_NO_ERROR      = 0,
  GWMCE_INVALID_MSG   = 1,
  GWMCE_OCC_NOT_FOUND = 2,
} GWMCError;

MsgChain * gwmc_create         ( );
void       gwmc_clear          ( MsgChain * msg );
void       gwmc_destroy        ( MsgChain ** msg );
void     __gwmc_destroy        ( MsgChain * msg );
void     * gwmc_read_data      ( MsgChain *msg, uint id, 
                                 char ** content_type, int occ, long *len );

int        gwmc_delete_data    ( MsgChain *msg, uint id, 
                                 const char * content_type, int occ, long *len);
  
int        gwmc_add_data       ( MsgChain *msg, uint id, 
                         const char* content_type, const char* data, long len );

MsgChain * gwmc_import         ( void      * buffer );
MsgChain * gwmc_importGWMessage( GWMessage * gwmsg  );

void     * gwmc_get_message_ptr( MsgChain *msg, long *len );
void       gwmc_debug          ( MsgChain * msg );

/* External Wrapper Helpers (For SWIG, PInvoke etc)
 */
void     __gwmc_read_data      ( MsgChain *msg, uint id, char ** content_type, 
		                         uchar_t **data, int occ, long *len );
void     __gwmc_get_message_ptr( MsgChain *msg, uchar_t **data, long *len );

#endif 

/* ------------------------------------------------------------------------ */
/* end */
