#ifndef XL_NETCOM
#define XL_NETCOM
/*
 * Network Communication Layer 
 *
 * Multiplexed Network Client/Server Core Engine
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 *
 * Goldwater is written and maintained by netFluid Technology Ltd.
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * -----------------------------------------------------------------------
 * $Id: gwneng.h,v 1.2 2003/03/29 12:21:11 csmith Exp $  $Date: 2003/03/29 12:21:11 $
 * -----------------------------------------------------------------------
 */
#include "gwnevent.h"

typedef enum { FREE_SESSION = 200, TERMINATE_SESSION, KEEP_CONNECTION, 
	       FINISHED, TERMINATE_SERVER, ENABLE_WRITE, DONE_WRITE, 
	       DISABLE_READ, ENABLE_READ, MDL_ERROR } Action;

/* 
 * Internal storage of module callback functions and Net deatails.
 * Some of these items are also found in module.h, but they are part of
 * a completely different layer and so we use our own copies.
 */
typedef struct {
  int          port;                    /* Port Number                   */
  int          socket;
  const char  *svcidstr;                /* Service ID String             */
  const char  *ipaddr;                  /* Host or IP to connect bind on */
  const char  *desc;                    /* Module Description            */
  Action (*read_handler)(int, void** ); /* Pointer to ReadData handler   */
  Action (*event_handler)(EventMsg*, void** ); /* Pointer to Event handler */

} NetService;

#define NETSVCEND    -1024

RetValue gwne_netcom_init       ( NetService *nsrv, int c, int m, int wblks );
RetValue gwne_netcom_deinit     ();
RetValue gwne_netcom_init_server( int nagle_off );
RetValue gwne_netcom_add_handler( NetService *ns );
void     gwne_netcom_poll       ( int loop_count, int allow_block );

/* helpers */
void _netcom_set_wrfd( int );
void _netcom_reset_wrfd( int );


#endif




