#ifndef _GW_REQ_H
#define _GW_REQ_H
/*
 * Goldwater Request Parameters
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 *
 * Goldwater is written and maintained by netFluid Technology Ltd.
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * ----------------------------------------------------------------------------
 * $Log: gwreq.h,v $
 * Revision 1.2  2003/03/29 12:21:11  csmith
 * cvs merge to 0.3.4
 *
 * Revision 1.20  2002/09/01 22:38:59  chris
 * added uuid generation code fom e2fsprogs-1.27
 *
 * Revision 1.19  2002/08/26 00:29:54  chris
 * More control for domain handling (like forcing the removal of spoolfiles)
 *
 * Revision 1.18  2002/08/04 19:36:03  chris
 * query_bb now allows scanning of a single domain's services.
 * API changes to allow replies to be sent back from dombridge.
 *
 * Revision 1.17  2002/06/16 23:04:04  chris
 * Only include uuid once.
 *
 * Revision 1.16  2002/06/15 19:32:05  chris
 * Message ID values are now 128 bit OSF DCE compatible uuid's.
 *
 * Revision 1.15  2002/06/15 18:54:39  chris
 * Domain Bridge additions
 *
 * Revision 1.14  2002/06/10 17:34:47  chris
 * All phlib includes have phlib/ prefix and gwlib includes have nothing.
 *
 * Revision 1.13  2002/06/04 16:35:45  chris
 * Introduce target_dom
 *
 * Revision 1.12  2002/05/12 14:03:13  chris
 * GPL licence headers etc
 *
 * Revision 1.11  2002/05/11 13:40:44  chris
 * Clean up - for Domain handling.
 *
 * Revision 1.10  2002/03/17 11:17:03  chris
 * Different sizes for service names and function names (service >= func)
 *
 * Revision 1.9  2002/03/12 22:51:03  chris
 * Added BBReqQueue support.  Currently defines a BBReqQueue entry for
 * each server, even though servers are grouped into classes and thus
 * will usually share a queue.
 *
 * Revision 1.8  2002/03/10 19:35:33  chris
 * Handles proper homed messages (initially for shutdown of servers).
 *
 * Revision 1.7  2001/12/26 22:57:43  chris
 * Changed all references to Imagepage Computing to netFluid Technology.
 *
 * Revision 1.6  2001/04/01 22:48:38  chris
 * Made MsgBlock data field MSGDSIZE bytes.  Currently initialised to 16384.
 *
 * Revision 1.5  2001/03/26 18:45:40  chris
 * Added SVR_SHUTDOWN_MSG.
 *
 * Revision 1.4  2001/03/26 12:51:07  chris
 * Removed chain_ident and added dest_srv_id.
 *
 * Revision 1.3  2001/03/22 14:47:38  chris
 * Moved flag typedefs into gwtypes.h.
 *
 * Revision 1.2  2001/03/22 01:24:13  chris
 * Changed all the flags.
 * Error codes occupy the first 8bits inclusive, giving 255 different
 * error codes.  These are NOT bit fields.  The remaining 24 bits *are*
 * bit fields.  This makes the error codes slightly more meaningful and
 * less bizarre when reported - besides, they were never considered as
 * bit fields anyway and were basically an unjustifiable waste!
 *
 * Revision 1.1  2001/03/19 16:08:22  chris
 * Initial revision
 *
 * ----------------------------------------------------------------------------
 */
#include <sys/types.h>

#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/shm.h>

#include <gwconfig.h>
#include <gwtypes.h>

#ifdef HAVE_UUID_UUID_H
/* Use OS supplied uuid... */
#  include <uuid/uuid.h>
#else
#  include <gwuuid.h>
#endif

#include <phlib/xl_types.h>
#include <phlib/xl_string.h>

#define SVC_NM_MAX    31     /* Service  Name Max */
#define FNC_NM_MAX    31     /* Function Name Max */

#ifndef MSXMAG
#  define MSGMAX 4056 /* TODO: THIS COULD BE ANY SIZE - DEP ON KERNEL! */
#endif

/* Message Types
 */
typedef enum {
  /* General Message Types */
  GWM_SVC_REQUEST  = 0x00000001,
  GWM_SVC_REPLY    = 0x00000002,
  GWM_DOM_REQUEST  = 0x00000003,
  GWM_DOM_REPLY    = 0x00000004,
  GWM_DOM_CONNECT  = 0x00000005,
  GWM_DOM_PING     = 0x00000006,

  /* Homed Message Types */
  GWHM_HALT_SERVER = 0x10000001,
} MsgType;

#define SPOOL_FILE_PREFIX "GWSF"
#define MSGDSIZE          16384

typedef struct {
  /* User properties */
  long            total_size;    /* data size       */
  long            bsize;         /* block size      */ 
  long            ident;         /* Main mtype      */
  long            destsrvid;     /* Server Id       */
  long            target_dom;    /* Target DomID    */
  MsgType         msgtype;       /* Type of message */
  char           *host;          /* gwportal host   */
  long            port;          /* gwportal port   */
  long            msgqid;        /* Req queue id    */
  long            r_msgqid;      /* Reply queue id  */
  /* org_r_msgqid does not change through the GWDBridge/NetServer path
   * and allows the final receipt of the reply to be routed to the clients
   * expecting reply queue.
   */
  long            org_r_msgqid;  /* Original Reply queue id  */
  uuid_t          msgid;         /* Message Identfr */
  long            clid;          /* Client id       */
  char           *spool_dir;     /* Spool Directory */
  char           *service;       /* Service for req */
  char           *function;      /* Service Funct   */
  GWMsgFlags      flags;         /* Flags           */

  /* Private properties */
  CString         cache;         /* Data Cache      */
  long            bytes_spooled; /* Bytes spooled.. */
  char            spoolfile[128];/* Spool file name */
  unsigned char  *spfileptr;     /* mmap spool file */
  GWMsgCtrlFlags  cflags;
} GWMsgCtrl;

typedef struct {
  long    servid;
  long    msgqid;
  long    ident;
  MsgType msgtype;
} GWHomedMsg;

typedef struct {
  long    ident;
  /* DO NOT MOVE THE FLAGS ITEM FROM THIS LOCATION - THE ADDRESS OF IT'S *
   * POSITION IS USED TO CALCULATE THE SIZE OF THE MESSAGE HEADER!!!!!!! *
   */
  long    flags;         /* Control flags */
  MsgType msgtype;       /* Type of message                      */
  long    total_size;    /* Total size of data message           */
  long    dest_srv_id;   /* ServerID    of the intended server (Homed/DBrg) */
  long    dest_dom_id;   /* DomainID    of the intended/origin server (DBrg)*/
  long    org_rep_msgqid;/* Reply queue ID of the originating client  (DBrg)*/
  long    rep_msgqid;    /* Reply message queue id               */
  uuid_t  msgid;         /* Message ID                           */
  long    clid;          /* Client ID                            */
  char    service [ SVC_NM_MAX+1 ]; /* Service being called    */
  char    function[ FNC_NM_MAX+1 ]; /* Function service maps to *//*REMOVE THIS*/
  long    dsize;       /* number of significant bytes in following data */
  char    data[ MSGDSIZE ];
} MsgBlock;

#define MsgBlockHdrSize ( sizeof(MsgBlock) - MSGDSIZE )

#endif

