/*
 * Goldwater Types
 *
 * Copyright 1996-2002 netFluid Technology Ltd
 *
 * Goldwater is written and maintained by netFluid Technology Ltd.
 * http://www.nfluid.co.uk
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * ----------------------------------------------------------------------------
 * $Log: gwtypes.h,v $
 * Revision 1.2  2003/03/29 12:21:11  csmith
 * cvs merge to 0.3.4
 *
 * Revision 1.30  2003/01/07 00:29:38  chris
 * Changed phlib linking as we've dropped /lib/phlib ext
 *
 * Revision 1.29  2003/01/05 22:26:34  chris
 * Lots of trickery to do superuser related tasks as quickly and safely as
 * possible before being permanently demoted to 'user'.
 *
 * Revision 1.28  2002/09/01 10:59:31  chris
 * Addition of namespaceing to service name.
 * Service may now be of the form this.is.a.namespace/and/this/is/the/service
 * Wildcards are NOT allowed in RPC type message destinations.
 *
 * Revision 1.27  2002/08/26 00:29:54  chris
 * More control for domain handling (like forcing the removal of spoolfiles)
 *
 * Revision 1.26  2002/08/04 19:36:03  chris
 * query_bb now allows scanning of a single domain's services.
 * API changes to allow replies to be sent back from dombridge.
 *
 * Revision 1.25  2002/07/28 18:27:12  chris
 * Further integration with netserver.
 *
 * Revision 1.24  2002/06/30 19:24:02  chris
 * Added GW_NTIMEOUT flag
 *
 * Revision 1.23  2002/06/16 11:40:35  chris
 * BB Client is now the Domain Bridge message tracking list.
 * New error codes and messages.
 *
 * Revision 1.22  2002/06/15 18:54:39  chris
 * Domain Bridge additions
 *
 * Revision 1.21  2002/06/11 21:52:58  chris
 * Enabled timed blocking in gwp_get_request and use this in dbridge to
 * prevent spinning and chewing cpu time.
 *
 * Revision 1.20  2002/05/30 19:37:34  chris
 * Create entries in BBClient table for each new client attaching to
 * application.
 *
 * Revision 1.19  2002/05/26 17:58:29  chris
 * Added GWS_EXPORTABLE flag
 *
 * Revision 1.18  2002/05/26 15:34:22  chris
 * Added GWC_FORWARD so the domain bridge is able to send the message
 * over the net without having to return a reply itself.
 *
 * Revision 1.17  2002/05/13 19:58:49  chris
 * Additional functionality to support the Domain Bridge.
 * Now allows __gw_get_request to 'not block' on request queue, and to
 * indicate that it returned because no messages were available.
 * More flags to control this behaviour (both control and private_user).
 *
 * Revision 1.16  2002/05/13 17:57:35  chris
 * Beginnings of NEW domain support.
 *
 * Revision 1.15  2002/05/12 22:59:02  chris
 * Added new error GWE_UNSTABLE - returned when GW detected to be in an
 * unstable state.
 *
 * Revision 1.14  2002/05/12 14:03:13  chris
 * GPL licence headers etc
 *
 * Revision 1.13  2002/04/14 00:02:26  chris
 * *** empty log message ***
 *
 * Revision 1.12  2002/03/10 19:35:33  chris
 * Handles proper homed messages (initially for shutdown of servers).
 *
 * Revision 1.11  2002/02/28 00:55:16  chris
 * Various bug fixes.
 *
 * Revision 1.10  2002/02/24 18:05:38  chris
 * Moved APPDIR to BBControl block instead of duplicating it for each and
 * every server.
 * Return GWE_APPERR and set gwusrerr when Services return failure.
 * Use new regstrings that describe the new XML based configuration.
 * Added new functions to support individual addition of service entries.
 * New Version 1.14
 *
 * Revision 1.9  2002/02/17 23:39:35  chris
 * Added extern int gwerror
 *
 * Revision 1.8  2002/02/17 22:54:36  chris
 * More error constants.
 *
 * Revision 1.7  2001/12/26 22:57:43  chris
 * Changed all references to Imagepage Computing to netFluid Technology.
 *
 * Revision 1.6  2001/03/30 22:17:09  chris
 * Added GWE_BADHDR
 *
 * Revision 1.5  2001/03/26 18:47:49  chris
 * Added MsgIdent types (for msgrcv mtype values) and flags for 'homed'
 * message architecture.
 *
 * Revision 1.4  2001/03/26 10:35:21  chris
 * Added GWP_PIFSSI flag
 *
 * ----------------------------------------------------------------------------
 */

#ifndef _GW_REQ_TYPES_H
#define _GW_REQ_TYPES_H

typedef enum {
  /* Error Return Flags (Inclusive values spanning bits b0..b7 [255 values] ) 
   */
  GWE_NOERROR = 0x00000000,
  GWE_NOFUNC  = 0x00000001, /* No function mapped to service     */
  GWE_TIMEOUT = 0x00000002, /* Service call timed out            */
  GWE_GENFAIL = 0x00000003, /* General Application Failure       */
  GWE_NOTCNCT = 0x00000004, /* Client not connected to Goldwater */
  GWE_READERR = 0x00000005, /* Read error                        */
  GWE_GWUAVAL = 0x00000006, /* Goldwater Server Unavailable      */
  GWE_DISCNCT = 0x00000007, /* Disconnection...                  */
  GWE_BADDATA = 0x00000008, /* Bad Data                          */
  GWE_BADCNFG = 0x00000009, /* Bad Config Entry                  */
  GWE_CFGNFND = 0x0000000a, /* Config not found                  */
  GWE_SYSFAIL = 0x0000000b, /* Internal System Failure           */
  GWE_NOSVC   = 0x0000000c, /* Service does not exist            */
  GWE_SBLOCKD = 0x0000000d, /* Service is blocked from access    */
  GWE_REPSENT = 0x0000000e, /* Reply has already been sent       */
  GWE_REPNSNT = 0x0000000f, /* Reply was not expected, so not snt*/
  GWE_BADHDR  = 0x00000010, /* Bad Header                        */
  GWE_SPLDIR  = 0x00000011, /* Failure to write to spool dir     */
  GWE_APPERR  = 0x00000012, /* Application Level Failure         */

  GWE_BADCFG  = 0x00000013, /* Not a valid GW Config file        */
  GWE_BCFGARC = 0x00000014, /* Config not compiled for this arch */
  GWE_CFGRERR = 0x00000015, /* Cannot read config file           */
  GWE_NOTUP   = 0x00000016, /* Goldwater is not running          */
  GWE_UNSTABLE= 0x00000017, /* Goldwater is unstable             */
  GWE_STILLUP = 0x00000018, /* Goldwater is already running      */
  GWE_CLTFAIL = 0x00000019, /* Client could not attach to GW     */
  GWE_BADDMVER= 0x0000001a, /* Bad Domain Message Version Number */
  GWE_BADIPCKY= 0x0000001b, /* Bad IPCKey config entry           */
  GWE_BADLOGF = 0x0000001c, /* Bad Logfile config entry          */
  GWE_BADDADDR= 0x0000001d, /* Bad Domain Address config entry   */
  GWE_BADNSPC = 0x0000001e, /* Bad Namespace                     */
  GWE_BADUSER = 0x0000001f, /* Bad Username                      */
  GWE_BADGROUP= 0x00000020, /* Bad Usergroup                     */
  GWE_SETUSER = 0x00000021, /* Error becoming user               */
  GWE_SETGROUP= 0x00000022, /* Error becoming group              */

  /* Internal System Errors */
  GWE_MSNDWBLK= 0x000000fe, /* Message Send would block          */

  GWE_ERRMASK = 0x000000ff, /* MASK TO GET ALL THE ERROR FLAGS   */


  /* User Control Flags     */
  GW_NOREPLY  = 0x00000100, /* No reply wanted. */
  GW_NOBLOCK  = 0x00000200, /* Don't block on getreply */
  GW_NTIMEOUT = 0x00000400, /* Do not time out request */

  /* Server -> client return flags */
  GWE_SVCUFAIL= 0x00001000, /* Service returned with FAILURE     */
  GWE_DOMFAIL = 0x00002000, /* Domain Failure                    */

#ifdef GW_INTERNAL_FLAGS
  /* Private internal flags */
  GWP_ASYNC   = 0x00100000, /* ASYNC request.  Spool reply until required. */
  GWP_DFILE   = 0x00200000, /* Msg passed by file                */
  GWP_REPLY   = 0x00400000, /* A reply Message                   */
  GWP_PHXRET  = 0x00800000, /* Reply to go straight to phoenix   */
  GWP_LRGMSG  = 0x01000000, /* Large Message, needs spooling     */
  GWP_REMOTE  = 0x02000000, /* Remote originator                 */
  GWP_TIAVM   = 0x04000000, /* New unprocessed message           */
  GWP_HOMDMSG = 0x08000000, /* A Homed Message...                */
  GWP_REQNBLK = 0x10000000, /* Do not block waiting for request  */
  GWP_REQTBLK = 0x20000000, /* Timed Block whilst wating for req */
  GWP_FROMDOM = 0x40000000  /* Message originated in a remote dom*/
#endif
} GWMsgFlags;

#ifdef GW_INTERNAL_FLAGS
#  define NETSERVER_CLASS 2
#endif

typedef enum {
  GWC_SPOOL   = 0x00000001,
  GWC_PORTAL  = 0x00000002,
  GWC_DONE    = 0x00000004,
  GWC_REPSENT = 0x00000008,
  GWC_RDONLY  = 0x00000010,
  GWC_NOBLOCK = 0x00000020,
  GWC_TIMEBLK = 0x00000040,
  GWC_NOREQM  = 0x00000080,
  GWC_FORWARD = 0x00000100, /* Request Forwarded to Another Server     */
  GWC_FORCERM = 0x00000200  /* Force removal of spool files on cleanup */
} GWMsgCtrlFlags;

typedef enum {
  HOMED_MSG_IDENT   = 0x20000000,
  MIN_REQ_LISTEN_ID = 0x20000000,
  MAX_REQ_LISTEN_ID = 0x2fffffff
} MsgIdent;

typedef enum {
  GWS_LOCAL_SERVICE  = 0x00000000,
  GWS_REMOTE_SERVICE = 0x00000001,
  GWS_EXPORTABLE     = 0x00000002
} ServiceFlags;

extern int gwerror;
extern int gwusrerr;

#endif

#ifndef _GW_REQ_LOG_H
#define _GW_REQ_LOG_H

typedef enum {
  /* General Log Bindings */
  LOG_ERROR   = 0x0001, 
  LOG_DATA    = 0x0002,
  LOG_FLOW    = 0x0004,
  LOG_DEBUG   = 0x0008,
  LOG_INFO    = 0x0010,
  LOG_AUDIT   = 0x0020,

  /* Internal Debug Bindings */
  LOG_INTDBG  = 0x0040,

  /* Internal Non-Maskable Bindings */
  LOG_SYSERR  = 0x1000,
  LOG_SYSINFO = 0x2000,
  LOG_SERIOUS = 0x4000,
  LOG_SYSWARN = 0x8000,

  /* Mask values */
  LOG_ALLSYS  = 0xf000,
  LOG_ALWAYS  = 0xffff

} LogTypes;

#endif

#ifndef _GW_REPLY_STATUS
#define _GW_REPLY_STATUS

typedef enum { 
  GWFailure    = 0, 
  GWSuccess    = 1, 

#ifdef GW_INTERNAL_FLAGS
  /* These are private - Not for use in user level code */
  GWDomFailure = 2 
#endif
} GWStatus;

#define MAXIPCKEY 32768

#endif

/* end */
